#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <string>
#include <vector>

#include "stitcher.h"

using simplestitcher::Stitcher;
using simplestitcher::StitcherOptions;

#ifdef WIN32
#define SIZE_T_FORMAT "%d"
#else
#define SIZE_T_FORMAT "%zd"
#endif

void usage(const char *name) {
  fprintf(stderr, "Usage: %s img1 img2 ...\n", name);
  exit(EXIT_FAILURE);
}

int main(int argc, char **argv) {
  if (argc < 2) {
    usage(argv[0]);
  }

  srand(static_cast<unsigned int>(time(NULL)));

  const char *requested_output_file = NULL;
  StitcherOptions options;
  std::vector<std::string> image_files;
  for (int i = 1; i < argc; i++) {
    if (strstr(argv[i], "--ransac_num_iterations") == argv[i]) {
      if (1 != sscanf(argv[i], "--ransac_num_iterations=" SIZE_T_FORMAT,
                      &options.ransac_options.num_iterations)) {
        fprintf(stderr, "Bad value for --ransac_num_iterations.\n");
        usage(argv[0]);
      }
    } else if (strstr(argv[i], "--ransac_sample_size") == argv[i]) {
      if (1 != sscanf(argv[i], "--ransac_sample_size=" SIZE_T_FORMAT,
                      &options.ransac_options.sample_size)) {
        fprintf(stderr, "Bad value for --ransac_sample_size.\n");
        usage(argv[0]);
      }
    } else if (strstr(argv[i], "--ransac_min_inliers") == argv[i]) {
      if (1 != sscanf(argv[i], "--ransac_min_inliers=" SIZE_T_FORMAT,
                      &options.ransac_options.min_inliers)) {
        fprintf(stderr, "Bad value for --ransac_min_inliers.\n");
        usage(argv[0]);
      }
    } else if (strstr(argv[i], "--ransac_inlier_tolerance") == argv[i]) {
      if (1 != sscanf(argv[i], "--ransac_inlier_tolerance=%f",
                      &options.ransac_options.inlier_tolerance)) {
        fprintf(stderr, "Bad value for --ransac_inlier_tolerance.\n");
        usage(argv[0]);
      }
    } else if (strstr(argv[i], "--descriptor_match_threshold") == argv[i]) {
      if (1 != sscanf(argv[i], "--descriptor_match_threshold=%f",
                      &options.descriptor_match_threshold)) {
        fprintf(stderr, "Bad value for --descriptor_match_threshold.\n");
        usage(argv[0]);
      }
    } else if (strstr(argv[i], "--output_file") == argv[i]) {
      requested_output_file = strchr(argv[i], '=');
      if (requested_output_file == NULL || requested_output_file[1] == '\0') {
        fprintf(stderr, "Bad value for --output_file\n");
        usage(argv[0]);
      }
      requested_output_file = requested_output_file + 1;
    } else {
      image_files.push_back(argv[i]);
    }
  }

  if (image_files.empty()) {
    fprintf(stderr, "Must specify at least one input file.\n");
    usage(argv[0]);
  }

  Stitcher stitcher(options);
  for (size_t i = 0; i < image_files.size(); i++) {
    if (!stitcher.AddImageFromFile(image_files[i])) {
      return EXIT_FAILURE;
    }
  }

  if (!stitcher.Stitch()){
    return EXIT_FAILURE;
  }

  char output_file[1024];
  snprintf(output_file, 1024, "stitched_%020ld.jpg",
           static_cast<unsigned long>(time(NULL)));
  if (requested_output_file == NULL) {
    requested_output_file = output_file;
  }
  if (!cv::imwrite(requested_output_file, stitcher.output())) {
    fprintf(stderr, "Unable to write image to output file \"%s\".\n",
            requested_output_file);
  }
  fprintf(stderr, "Output written to \"%s\".\n", requested_output_file);

  return EXIT_SUCCESS;
}
