#ifndef SIMPLESTITCHER_STITCHER_H
#define SIMPLESTITCHER_STITCHER_H

#include <string>
#include <vector>

#include <opencv/cxcore.h>
#include <opencv/cv.h>
#include <opencv/highgui.h>

#include "util.h"

namespace simplestitcher {
struct RansacOptions {
  RansacOptions()
    : num_iterations(500000),
      sample_size(5),
      min_inliers(20),
      inlier_tolerance(2.0) {}

  size_t num_iterations;
  size_t sample_size;
  size_t min_inliers;
  float inlier_tolerance;
};

struct StitcherOptions {
  StitcherOptions()
    : descriptor_match_threshold(0.1) {}

  RansacOptions ransac_options;
  float descriptor_match_threshold;
};

class Stitcher {
 public:
  Stitcher(const StitcherOptions& options)
    : options_(options) {}

  void AddImage(const cv::Mat& image);
  bool AddImageFromFile(const std::string& filename);

  bool Stitch();

  const cv::Mat& output() const {
    return output_;
  }

  const std::vector<cv::Mat> images() const {
    return images_;
  }

 private:
  StitcherOptions options_;
  cv::Mat output_;
  std::vector<cv::Mat> images_;

  DISALLOW_COPY_AND_ASSIGN(Stitcher);
};
}  // namespace simplestitcher

#endif // SIMPLESTITCHER_STITCHER_H
