#ifndef CATCHMENT_UTIL_H
#define CATCHMENT_UTIL_H

#define DISALLOW_COPY_AND_ASSIGN(Classname)  \
  Classname(const Classname& other); \
  void operator =(const Classname& other)

template<typename T>
class scoped_ptr {
 public:
  scoped_ptr(T* p) : p_(p) {}
  scoped_ptr() : p_(NULL) {}

  void reset(T* p) {
    if (p_) delete p_;
    p_ = p;
  }

  T* release() {
    T* p = p_;
    p_ = NULL;
    return p;
  }

  T* operator->() const {
    return p_;
  }

  T& operator*() const {
    return *p_;
  }

 private:
  T* p_;

  DISALLOW_COPY_AND_ASSIGN(scoped_ptr);
};

template <typename T, typename U>
bool ContainsKey(const T& container, const U& key) {
  return container.find(key) != container.end();
}
#endif // CATCHMENT_UTIL_H
